package arzumify.located.impl;

import arzumify.located.api.Provider;
import arzumify.located.math.Vec3i;

import java.util.HashSet;
import java.util.Set;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class MatchMaker<P extends Provider> {
    public MatchMaker() {
    }

    private final Map<String, Map<Vec3i, P>> globalMap = new ConcurrentHashMap<>();

    public void Add(P provider) {
        globalMap.putIfAbsent(provider.dimension(), new ConcurrentHashMap<>());
        for (var point : provider.points()) {
            globalMap.get(provider.dimension()).put(point, provider);
        }
    }

    private static final Vec3i[] directions = new Vec3i[]{
            new Vec3i(1, 0, 0),
            new Vec3i(-1, 0, 0),
            new Vec3i(0, 1, 0),
            new Vec3i(0, -1, 0),
            new Vec3i(0, 0, 1),
            new Vec3i(0, 0, -1)
    };

    public Set<P> Search(P provider) {
        var matches = new HashSet<P>();
        var dimension = provider.dimension();
        for (var direction : directions) {
            for (var point : provider.points()) {
                var offsetPoint = point.add(direction);
                if (globalMap.get(dimension).containsKey(offsetPoint)) {
                    var match = globalMap.get(dimension).get(offsetPoint);
                    if (match != provider) {
                        matches.add(match);
                    }
                }
            }
        }
        return matches;
    }

    public void Remove(P provider) {
        for (var point : provider.points()) {
            globalMap.get(provider.dimension()).remove(point);
        }
    }
}