import arzumify.located.impl.MatchMaker;
import arzumify.located.math.Vec3i;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;

import java.util.HashSet;
import java.util.Set;

import static org.junit.jupiter.api.Assertions.*;

@Timeout(1)
public class IntegerPresenceTests {
    @Nested
    class TestMatchMaker {
        private final MatchMaker<ExampleIntegerPresence> matchMaker = new MatchMaker<>();
        private final ExampleIntegerPresence presence = new ExampleIntegerPresence(new Vec3i(0, 0, 0));
        private final ExampleIntegerPresence presence2 = new ExampleIntegerPresence(new Vec3i(1, 0, 0));

        @Test
        public void testMatchMaker() {
            System.out.println("Testing match maker");
            matchMaker.Add(presence);
            matchMaker.Add(presence2);
            var matches = matchMaker.Search(presence);
            assertEquals(1, matches.size());
            System.out.println(matches);
            assertTrue(matches.contains(presence2));
        }
    }

    @Nested
    class TestMatchMakerManyBasePositions {
        private final MatchMaker<ExampleIntegerPresence> matchMaker = new MatchMaker<>();
        private final ExampleIntegerPresence presence = new ExampleIntegerPresence(new Vec3i(0, 0, 0), new Vec3i(1, 0, 0));
        private final ExampleIntegerPresence presence2 = new ExampleIntegerPresence(new Vec3i(2, 0, 0), new Vec3i(3, 0, 0));

        @Test
        public void testMatchMakerManyBasePositions() {
            System.out.println("Testing match maker many base positions");
            matchMaker.Add(presence);
            matchMaker.Add(presence2);
            var matches = matchMaker.Search(presence);
            assertEquals(1, matches.size());
            assertTrue(matches.contains(presence2));
        }
    }

    @Nested
    class StressTestBasePositions {
        private final MatchMaker<ExampleIntegerPresence> matchMaker = new MatchMaker<>();
        private final Set<ExampleIntegerPresence> presenceSet = new HashSet<>();
        private final ExampleIntegerPresence originalPresence = new ExampleIntegerPresence(GenerateVec3isFromArea(new Vec3i(0, 0, 0), new Vec3i(0, 0, 50)));

        public StressTestBasePositions() {
            for (int i = 0; i < 50; i++) {
                var presence = new ExampleIntegerPresence(new Vec3i(0, 1, i));
                presenceSet.add(presence);
            }
        }

        @Test
        @Timeout(10)
        public void stressTestBasePositions() {
            for (ExampleIntegerPresence presence : presenceSet) {
                matchMaker.Add(presence);
            }
            var matches = matchMaker.Search(originalPresence);
            assertEquals(presenceSet.size(), matches.size());
            assertFalse(matches.contains(originalPresence));
        }
    }

    private Vec3i[] GenerateVec3isFromArea(Vec3i corner1, Vec3i corner2) {
        var vec3is = new HashSet<Vec3i>();

        int minX = Math.min(corner1.x(), corner2.x());
        int maxX = Math.max(corner1.x(), corner2.x());
        int minY = Math.min(corner1.y(), corner2.y());
        int maxY = Math.max(corner1.y(), corner2.y());
        int minZ = Math.min(corner1.z(), corner2.z());
        int maxZ = Math.max(corner1.z(), corner2.z());

        for (int x = minX; x <= maxX; x++) {
            for (int y = minY; y <= maxY; y++) {
                for (int z = minZ; z <= maxZ; z++) {
                    vec3is.add(new Vec3i(x, y, z));
                }
            }
        }

        return vec3is.toArray(new Vec3i[0]);
    }

}
